//--------------------------
var __gMaps = null; 
function L2SGMaps(key, mapId, display, zoom, showControls, erroFlow) { 
  this.directionsService = false;
  this.directionsDisplay = false;
  this.markees = new Array();
  this.infoWindows = new Array();
  this.linePaths = new Array();
  this.layers = new Array();
  this.showControls = parseBoolean(showControls);
  this.mapDiv = display;
  this.zoomLevel = zoom ? parseFloat(zoom) : 14;
  this.onErrorEvent = erroFlow;
  if(!mapId) {
    mapId = 'DEMO_MAP_ID';
  }
  var disableDefUI = !showControls;
  var mapOptions = {
		disableDefaultUI: disableDefUI,
		zoom: this.zoomLevel,
		numZoomLevels: 23,
		MAX_ZOOM_LEVEL: 22,
		mapTypeId: google.maps.MapTypeId.ROADMAP,
		mapId: mapId
  }

  if(showControls && (typeof(showControls) == 'string')) {
		showControls = showControls.toUpperCase();
		if(showControls.indexOf('STREETVIEW') >= 0) {
			mapOptions.streetViewControl = true;
			mapOptions.disableDefaultUI = true;
			streetViewControlOptions: {
				position: google.maps.ControlPosition.TOP_CENTER
			}
		}
		if(showControls.indexOf('ROTATE') >= 0) {
			mapOptions.rotateControl = true;
			mapOptions.disableDefaultUI = true;
		}
		if(showControls.indexOf('FULLSCREEN') >= 0) {
			mapOptions.fullscreenControl = true;
			mapOptions.disableDefaultUI = true;
		}
		
  }
  
  this.Map = new google.maps.Map(this.mapDiv, mapOptions);
  this.Geo = new google.maps.Geocoder();
  
  
  this.key = key;
  this.sensor = false;
  __gMaps = this;
}

L2SGMaps.prototype.doErrorEvent = function (status, description) {
	if(!this.onErrorEvent) {
		alert(status + ': ' + description);
	}
	else {
		callFlow(this.onErrorEvent, [status, description]);
	}
};

L2SGMaps.prototype.coordsDistance = function (lat1, lon1, lat2, lon2) {
  var R = 6371000;
  lat1 = toRad(lat1)
  lon1 = toRad(lon1);
  lat2 = toRad(lat2)
  lon2 = toRad(lon2);
  dLat = lat2 - lat1;
  dLon = lon2 - lon1;

  var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
          Math.cos(lat1) * Math.cos(lat2) * 
          Math.sin(dLon/2) * Math.sin(dLon/2);
  var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
  var d = R * c;
  return Math.ceil(d);
}

L2SGMaps.prototype.extractCoords = function (txt) {
  if(typeof(txt) == 'undefined') return null;

	//Verifica se já é um objeto do Google Maps e retorna o mesmo
  if ((typeof(txt) == 'object') && (typeof(txt.lat)  == 'function')) {
		return txt;
  }
    var lat = NaN;
    var lng = NaN;
    var latlngStr = txt.split(";", 2);
    if (latlngStr.length == 2) {
      lat = this.isDMSCoord(latlngStr[0]) ? this.convertDMStoDecimal(latlngStr[0]) : parseFloat(latlngStr[0].replace(',', '.'));
      lng = this.isDMSCoord(latlngStr[1]) ? this.convertDMStoDecimal(latlngStr[1]) : parseFloat(latlngStr[1].replace(',', '.'));
    }
    else {
      latlngStr = txt.split(",", 2);
	    if (latlngStr.length == 2) {
	      lat = this.isDMSCoord(latlngStr[0]) ? this.convertDMStoDecimal(latlngStr[0]) : parseFloat(latlngStr[0].replace(',', '.'));
        lng = this.isDMSCoord(latlngStr[1]) ? this.convertDMStoDecimal(latlngStr[1]) : parseFloat(latlngStr[1].replace(',', '.'));
	    }
    }

    if(isNaN(lat) || isNaN(lng)) {
      return false;
    }
    else {
      return new google.maps.LatLng(lat, lng);
    }	
}

L2SGMaps.prototype.isDMSCoord = function(str) {
  str = "" + str + " ";//Força a conversão para string      
  if (str.length > 7) {
    return (str.indexOf('°') != -1) || (str.indexOf("'") != -1) || (str.indexOf('"') != -1) || ("SNEWsnew".indexOf(str[0]) != -1)  || ("SNEWsnew".indexOf(str[str.length - 2]) != -1);    
  } else {  
    return false;
  }
}

L2SGMaps.prototype.convertDMStoDecimal = function(str) {
  str = "" + str;
  str = str.toUpperCase() + ' ';
  var degree, minutes, seconds = 0;
  var partCount = 0;
  var buffer = '';
  for(var i = 0; i < str.length; i++) {                  
    if ((!isEmptyString(buffer)) && (('°\'"` SNEW'.indexOf(str.charAt(i)) != -1)) || (str.charCodeAt(i) == 186)) {
      buffer = trimString(buffer);      
      switch (partCount) {
        case 0: degree = buffer; break;
        case 1: minutes = buffer; break;
        case 2: seconds = buffer; break;
        case 3: if (buffer == 'S' || buffer == 'W') {
          degree = degree * -1;
        }
          break;
      }
      buffer = '';
      partCount++;
    }  else {
      buffer += str.charAt(i);
    }
  }
  
  var latsign = (degree > 0)?1:-1;
  //Degree
  var absdlat = Math.abs(Math.round(degree * 1000000.));
  if(absdlat > (90 * 1000000)) { 
    throw 'Degrees Latitude must be in the range of -90 to 90.'; 
  }
  //Minutes
  minutes = Math.abs(Math.round(minutes * 1000000.)/1000000);  //integer
  var absmlat = Math.abs(Math.round(minutes * 1000000.));  //integer
  if(absmlat >= (60 * 1000000)) {  
    throw 'Minutes Latitude must be in the range of 0 to 59. ';
  }
  //Seconds
  seconds = Math.abs(Math.round(seconds * 1000000.)/1000000);
  var absslat = Math.abs(Math.round(seconds * 1000000.));// Note: kept as big integer for now, even if submitted as decimal
  
  //Error checks
  if(absslat > (59.99999999 * 1000000)) {  
    throw 'Minutes Latitude must be 0 or greater \n and less than 60.';
  }
  return Math.round(absdlat + (absmlat/60.) + (absslat/3600.) ) * latsign/1000000;
}

L2SGMaps.prototype.getAddressCoord = function (address) {
  address = removeSpecialChars(address);
  var baseURL = window.location.href;
  baseURL = baseURL.substring(0, baseURL.lastIndexOf('/') + 1);
  if(baseURL.indexOf('l2s/maps') == -1) {
	baseURL += 'l2s/maps/';
  }

  var url = baseURL + 'gproxy.jsp?service=geocode&resultType=json&address=' + URLEncode(address);
  url += '&key=' + this.key;
  var http = getHTTPObject();
  http.open('GET', url, false);
  http.send(null);
  var content = http.responseText;
  if(content.substr(0, 6) == '[ERRO:') {
    throw content;
  }
  var responseObj = eval('(' + content + ')');
  if(responseObj.status != "OK") {
    this.doErrorEvent(responseObj.status, address);
  }
  var l = responseObj.results[0].geometry.location;
  var lat = l.lat;
  var lng = l.lng;
  return new google.maps.LatLng(lat, lng);
}

L2SGMaps.prototype.drawLineCoords = function (lineName, fromLat, fromLng, toLat, toLng, strokeColor, strokeWeight, strokeOpacity, onClickEvent, onDblClickEvent) {
    strokeOpacity = parseFloat(strokeOpacity) / 100;
    var lineCoordinates = [
          new google.maps.LatLng(fromLat, fromLng),
          new google.maps.LatLng(toLat, toLng)
    ];
  
    var linePath = new google.maps.Polyline({
      path: lineCoordinates,
      strokeColor: strokeColor,
      strokeOpacity: strokeOpacity,
      strokeWeight: strokeWeight
    });

    linePath.setMap(this.Map);
	this.linePaths.push(linePath);

	if(onClickEvent) {
		google.maps.event.addListener(linePath, 'click', function(event) { callFlow(onClickEvent, [lineName, event.latLng.lat(), event.latLng.lng()]) });
	}
  
	if(onDblClickEvent) {
		google.maps.event.addListener(linePath, 'dblclick', function(event) { callFlow(onDblClickEvent, [lineName, event.latLng.lat(), event.latLng.lng()]) });
	}
}
  
L2SGMaps.prototype.coordsDistanceEx = function (from, to) {
    var lat1 = parseFloat(from.lat());
	var lon1 = parseFloat(from.lng());
    var lat2 = parseFloat(to.lat());
	var lon2 = parseFloat(to.lng());
	
    return this.coordsDistance(lat1, lon1, lat2, lon2);
}

L2SGMaps.prototype.drawLine = function (itemId, from, to, strokeColor, strokeWeight, strokeOpacity, onClickEvent, onDblClickEvent) {  
  this.lineCoords = new Array();
  var fromCoords = this.extractCoords(from);
  if(!fromCoords) {
    fromCoords = this.getAddressCoord(from);
  }

  var toCoords = this.extractCoords(to);
  if(!toCoords) {
    toCoords = this.getAddressCoord(to);
  }

  if(toCoords && fromCoords) {
	this.drawLineCoords(itemId, fromCoords.lat(), fromCoords.lng(), toCoords.lat(), toCoords.lng(), strokeColor, strokeWeight, strokeOpacity, onClickEvent, onDblClickEvent);
  }
  else {
    this.doErrorEvent(404, 'Endereço não encontrado. Origem: "' + from + '", destino "' + to + '".'); 
  }
}

L2SGMaps.prototype.drawCircle = function (itemId, center, radius, fillColor, fillOpacity, strokeColor, strokeWeight, strokeOpacity, onClick, onDblClick) {
  var hasEvent = parseBoolean(onClick) || parseBoolean(onDblClick);
  var centerCoords = this.extractCoords(center);
  if(!centerCoords) {
    centerCoords = this.getAddressCoord(center);
  }
  radius = parseFloat(radius);
  fillOpacity = parseFloat(fillOpacity) / 100;
  strokeOpacity = parseFloat(strokeOpacity) / 100;
  var circOptions = {
	center: centerCoords,
    clickable: hasEvent,
	radius: radius,
    fillColor: fillColor,
	fillOpacity: fillOpacity,
	strokeColor: strokeColor, 
	strokeWeight: strokeWeight,
	strokeOpacity: strokeOpacity
  }
    
  var circ = new google.maps.Circle(circOptions);
  circ.setMap(this.Map);
  this.linePaths.push(circ);
  
  if(onClick) {
    google.maps.event.addListener(circ, 'click', function(event) { callFlow(onClick, [itemId, event.latLng.lat(), event.latLng.lng()]) });
  }
  
  if(onDblClick) {
    google.maps.event.addListener(circ, 'dblclick', function(event) { callFlow(onDblClick, [itemId, event.latLng.lat(), event.latLng.lng()]) });
  }    
}

L2SGMaps.prototype.drawPolygon = function (itemId, points, fillColor, fillOpacity, strokeColor, strokeWeight, strokeOpacity, onClick, onDblClick) {  
  var gPoints = new google.maps.MVCArray();
  var p;
  if((typeof points) == 'string') {
	points = points.split('\n');
  }
  for(var i = 0; i < points.length; i++) {
    p = points[i];
    if(!this.extractCoords(p)) {
      p = this.getAddressCoord(p);
    }
    gPoints.push(this.extractCoords(p));
  }
  this.internalDrawPolygon(itemId, gPoints, fillColor, fillOpacity, strokeColor, strokeWeight, strokeOpacity, onClick, onDblClick);
}

L2SGMaps.prototype.addInfoWindow = function (name, address, textContent) {
	var coords = this.extractCoords(address);
	if(!coords) {
		coords = this.getAddressCoord(address);
	}
	var infoWindow = new google.maps.InfoWindow({
		content: textContent, 
		position: coords
		});
	__gMaps.infoWindows.push(infoWindow);
	infoWindow.open(this.Map);
}

L2SGMaps.prototype.internalDrawPolygon = function(itemId, points, fillColor, fillOpacity, strokeColor, strokeWeight, strokeOpacity, onClick, onDblClick) {  
  var hasEvent = parseBoolean(onClick) || parseBoolean(onDblClick);
  fillOpacity = parseFloat(fillOpacity) / 100;
  strokeOpacity = parseFloat(strokeOpacity) / 100;

	var polygonOptions = {
		clickable: hasEvent,
		map: this.Map,
		fillColor: fillColor,
		fillOpacity: fillOpacity,
		strokeColor: strokeColor, 
		strokeWeight: strokeWeight,
		strokeOpacity: strokeOpacity,
		paths: points
	}
  
  var polygon = new google.maps.Polygon(polygonOptions);
  polygon.setMap(this.Map);
  polygon.__itemId = itemId;
  
  this.linePaths.push(polygon);
  
  if(onClick) {
    google.maps.event.addListener(polygon, 'click', function(event) { callFlow(onClick, [itemId, event.latLng.lat(), event.latLng.lng()]) });
  }
  
	if(onDblClick) {
		google.maps.event.addListener(polygon, 'dblclick', function(event) { callFlow(onDblClick, [itemId, event.latLng.lat(), event.latLng.lng()]) });
	}
}

L2SGMaps.prototype.drawPolyline = function (itemId, points, strokeColor, strokeWeight, strokeOpacity, onClick, onDblClick) {  
  var gPoints = new google.maps.MVCArray();
  var p;
  for(var i = 0; i < points.length; i++) {
    p = points[i];
    if(!this.extractCoords(p)) {
      p = this.getAddressCoord(p);
    }
    gPoints.push(this.extractCoords(p));
  }
  this.internalDrawPolyline(itemId, gPoints, strokeColor, strokeWeight, strokeOpacity, onClick, onDblClick);
}

L2SGMaps.prototype.internalDrawPolyline = function (itemId, points, strokeColor, strokeWeight, strokeOpacity, onClick, onDblClick) {  
	var hasEvent = parseBoolean(onClick) || parseBoolean(onDblClick);
	strokeOpacity = parseFloat(strokeOpacity) / 100;

	var polylineOptions = {
		clickable: hasEvent,
		map: this.Map,
		strokeColor: strokeColor, 
		strokeWeight: strokeWeight,
		strokeOpacity: strokeOpacity,
		path: points
	}
  
  var polyline = new google.maps.Polyline(polylineOptions);
  polyline.setMap(this.Map);
  this.linePaths.push(polyline);
  
  if(onClick) {
    google.maps.event.addListener(polyline, 'click', function(event) { callFlow(onClick, [itemId, event.latLng.lat(), event.latLng.lng()]) });
  }

  if(onDblClick) {
    google.maps.event.addListener(polyline, 'dblclick', function(event) { callFlow(onDblClick, [itemId, event.latLng.lat(), event.latLng.lng()]) });
  }    
}

L2SGMaps.prototype.drawTriangle = function (itemId, a, b, c, fillColor, fillOpacity, strokeColor, strokeWeight, strokeOpacity, onClick, onDblClick) {  
  var points = new google.maps.MVCArray();
  if(!this.extractCoords(a)) {
    a = this.getAddressCoord(a);
  }
  points.push(this.extractCoords(a));
  
  if(!this.extractCoords(b)) {
    b = this.getAddressCoord(b);
  }
  points.push(this.extractCoords(b));
  
  if(!this.extractCoords(c)) {
    c = this.getAddressCoord(c);
  }
  points.push(this.extractCoords(c));
  
  this.internalDrawPolygon(itemId, points, fillColor, fillOpacity, strokeColor, strokeWeight, strokeOpacity, onClick, onDblClick);
}

L2SGMaps.prototype.showAddress = function (itemId, address, textContent) {
  this.Geo.geocode({'address': address}, 
    function(results, status){  
		var iconTitle = textContent;
		if(textContent.indexOf('|') > 0) {
			iconTitle = iconTitle.substring(0, iconTitle.indexOf('|'));
			textContent = textContent.substring(textContent.indexOf('|')+1);
		}

      if (status == google.maps.GeocoderStatus.OK) {
        __gMaps.Map.setCenter(results[0].geometry.location);
        var marker = new google.maps.marker.AdvancedMarkerElement({
              map: __gMaps.Map, 
			  title: iconTitle,
			  visible: true,
              position: results[0].geometry.location,
			  _iconName: itemId
        });
		__gMaps.markees.push(marker);

		if(textContent) {
			var infoWindow = new google.maps.InfoWindow({
				content: textContent
			});
			
			infoWindow.open(__gMaps.Map, marker);
			google.maps.event.addListener(marker, 'click', function() {
				__gMaps.closeAllInfoWindows();
				infoWindow.open(__gMaps.Map, marker);
			});

			__gMaps.infoWindows.push(infoWindow);
		}
	  } else {
		__gMaps.doErrorEvent(status, address);
	  }
    }
  );
}

L2SGMaps.prototype.addTextMark = function (name, title, address, textContent, onClick, onDblClick, onDragStart, onDragEnd) {
	var g = this;
	generateIcon(title, function(src) {
		g.addIcon(name, src, address, textContent, onClick, onDblClick, onDragStart, onDragEnd);
	});
}

L2SGMaps.prototype.addIcon = function (name, iconURL, address, textContent, onClick, onDblClick, onDragStart, onDragEnd) {
	if(name == '') {
		name = this.markees.length + '_' + iconURL; 
	}
	var iconLatLng = this.extractCoords(address);
	if(!iconLatLng) {
		iconLatLng = this.getAddressCoord(address);
	}
	var iconTitle = textContent;
	if(textContent.indexOf('|') > 0) {
		iconTitle = iconTitle.substring(0, iconTitle.indexOf('|'));
		textContent = textContent.substring(textContent.indexOf('|')+1);
	}

	if(iconLatLng) {//icon: iconURL, _iconName: name
		var map = this.Map;//
		var iconMarker = new google.maps.marker.AdvancedMarkerElement({
                                            map,
                                            position: iconLatLng,
                                            gmpDraggable: isDraggable,
                                            title: iconTitle
                                            
                                            });
    
	var infoWindow = false;
	if(!textContent) {
		showInfo = false;
		textContent = '&nbsp;';
	} else {
		infoWindow = new google.maps.InfoWindow({
                                                  content: textContent
                                                });
	}
	
	var isDraggable = !isNullOrEmpty(onDragStart) || !isNullOrEmpty(onDragEnd);
	//OnClick
	if(!isNullOrEmpty(onClick)) {
		google.maps.event.addListener(iconMarker, 'click', function(event) {
									callFlow(onClick, [name, event.latLng.lat(), event.latLng.lng()]);
                                  });
	} else {
	    if(!isDraggable) {
			google.maps.event.addListener(iconMarker, 'click', function() {
									if(infoWindow) {
										__gMaps.closeAllInfoWindows();
										infoWindow.open(__gMaps.Map, iconMarker);
									}
                                  });
		}
    }	
	
	//OnDblClick
	if(!isNullOrEmpty(onDblClick)) {
		google.maps.event.addListener(iconMarker, 'dblclick', function(event) {
									callFlow(onDblClick, [name, event.latLng.lat(), event.latLng.lng()]);
                                  });
	}

	//OnDragStart
	if(!isNullOrEmpty(onDragStart)) {
		google.maps.event.addListener(iconMarker, 'dragstart', function(event) {
									callFlow(onDragStart, [name, event.latLng.lat(), event.latLng.lng()]);
                                  });
	}
	
	//OnDragEnd
	if(!isNullOrEmpty(onDragEnd)) {
		google.maps.event.addListener(iconMarker, 'dragend', function(event) {
									callFlow(onDragEnd, [name, event.latLng.lat(), event.latLng.lng()]);
                                  });
	}

	this.markees.push(iconMarker);
	if(infoWindow) {
		this.infoWindows.push(infoWindow);
		iconMarker.infoWindow = infoWindow;
	}
	return name;
	}
	else {
		this.doErrorEvent(500, 'Formato do endereço não suportado, ou local não encontrado.'); 
	}
} 

L2SGMaps.prototype.getIconByName = function (name) {
  for(var i = 0; i < this.markees.length; i++) {
    if(this.markees[i]._iconName == name) return this.markees[i];
  }
  this.doErrorEvent(404, 'O icone não foi localizado: ' + name);
}

L2SGMaps.prototype.moveIcon = function (name, address) {
  var icon = this.getIconByName(name);
  if (icon) { 
    var iconLatLng = this.extractCoords(address);
    if(!iconLatLng) {
      iconLatLng = this.getAddressCoord(address);
    }    
    icon.setPosition(iconLatLng);
  }
}

L2SGMaps.prototype.showIconPopup = function (name, show) {
	var icon = this.getIconByName(name);
	if (icon) {
		if(parseBoolean(show)) {
			icon.infoWindow.open(this.Map, icon);
		} else {
			icon.infoWindow.close();
		}
	}
}

L2SGMaps.prototype.closeAllInfoWindows = function() {
	for(var i = 0; i < this.infoWindows.length; i++) {
		var infoW = this.infoWindows[i];
		infoW.close();
	};
}

L2SGMaps.prototype.setIconPopupContent = function (name, content) {
	var icon = this.getIconByName(name);
	if (icon) {
		icon.infoWindow.setContent(content);
	}
}

L2SGMaps.prototype.removeIcon = function (name) {
	var icon = this.getIconByName(name);
	if(icon) {
		if(icon.infoWindow) {
		icon.infoWindow.close();
		}  
		icon.setVisible(false);
		icon._iconName = null;
	}
}

L2SGMaps.prototype.drivingDirections = function (from, to, optimizePath, lineColor, onSuccessEvent, onErrorEvent) {
  var wPoints = null;
  if((typeof(to) == 'array') || (typeof(to) == 'object')) {
    wPoints = new Array();
    var p;
    for(var i = 0; i < to.length - 1; i++) {
      p = to[i];
      if(this.extractCoords(p)) {
        p = this.extractCoords(p);
      }

      var wP = { location: p };
      wPoints.push(wP);
    }
    //Ultimo item é o destino final
    p = to.pop();
    if(this.extractCoords(p)) {
      p = this.extractCoords(p);
    }    
    to = p;
  } else {
    //
  }
  
  var dService = new google.maps.DirectionsService();
  var dRenderer; 
  if((lineColor != null) && (lineColor != '')) {
    dRenderer = new google.maps.DirectionsRenderer({polylineOptions: {strokeColor: lineColor} }); 
  } else {
    dRenderer = new google.maps.DirectionsRenderer(); 
  }
  if(!this.directions) this.directions = new Array();  
  this.directions.push({ service: dService, render: dRenderer});
    
  var request = {
        origin: from, 
        waypoints: wPoints,
        destination: to,
        optimizeWaypoints: optimizePath,
		travelMode: google.maps.DirectionsTravelMode.DRIVING
      };

  dRenderer.setMap(this.Map);

  dService.route(request, function(response, status) {
      if (status == google.maps.DirectionsStatus.OK) {
        dRenderer.setDirections(response);

		if(onSuccessEvent) {
		  var route = response.routes[0];
		  var distance = 0;
		  var duration = 0;
		  for(var i = 0; i < route.legs.length; i++) {
		    distance += route.legs[i].distance.value;
			duration += route.legs[i].duration.value;;
		  }
		  callFlow(onSuccessEvent, [distance, duration]);
		}
      } else {
		if(onErrorEvent) {
			callFlow(onErrorEvent, [status, response, from, to]);
		} else {
			__gMaps.doErrorEvent(status, response);
		}
	  }
    });
}

L2SGMaps.prototype.addMouseListener = function(onClick, onDblClick, onReverseClick) {
	if(onClick && !isNull(onClick)) {
		google.maps.event.addListener(this.Map, 'click', function(event) { callFlow(onClick, [event.latLng.lat(), event.latLng.lng()]) });
	}

	if(onDblClick && !isNull(onDblClick)) {
		google.maps.event.addListener(this.Map, 'dblclick', function(event) { callFlow(onDblClick, [event.latLng.lat(), event.latLng.lng()]) });

		var newOptions = {
			disableDoubleClickZoom: true
		}
		this.Map.setOptions(newOptions);
	} else {
		var newOptions = {
			disableDoubleClickZoom: false
		}
		this.Map.setOptions(newOptions);
	}  

	if(onReverseClick && !isNull(onReverseClick)) {
		google.maps.event.addListener(this.Map, 'rightclick', function(event) { callFlow(onReverseClick, [event.latLng.lat(), event.latLng.lng()]) });
	}
}

L2SGMaps.prototype.loadKmlLayer = function (name, src, onClick) {
	var kmlLayer = new google.maps.KmlLayer(src, {
	  suppressInfoWindows: true,
	  preserveViewport: true,
	  map: this.Map
	});

	kmlLayer.__layer_name = name;
	if(onClick) {
		google.maps.event.addListener(kmlLayer, 'click', function(event) {
			callFlow(onClick, [name, event.latLng.lat(), event.latLng.lng(), event.featureData.id, event.featureData.name, event.featureData]) 
		});
	}
	this.layers.push(kmlLayer);
};

L2SGMaps.prototype.showLayer = function(name, show) {
	for(var idx in this.layers) {
		var layer = this.layers[idx];
		if(layer.__layer_name == name) {
			layer.setMap(show ? this.Map : null);
			break;
		}
	}
}

L2SGMaps.prototype.clear = function () {
  this.Map.setOptions({ noClear: false });
  //Remove os pontos/marcas
  while (this.markees.length > 0) {
    var m = this.markees.pop();
    m.setVisible(false);
  }
  //Remove os dialogos
  while (this.infoWindows.length > 0) {
    var i = this.infoWindows.pop();
	i.close();
  }
  //Remove as rotas unicas
  if(this.directionsDisplay) {
    this.directionsDisplay.setMap(null);
	this.directionsDisplay = false;
  }

	//Remove as rotas unicas
	if(this.directions) {
		while (this.directions.length > 0) {
			var d = this.directions.pop();
			d.render.setMap(null);
		}
		this.directions = false;
	}

	//Remove as linhas
	while (this.linePaths.length > 0) {
		var i = this.linePaths.pop();
		i.setMap(null);
	}

	//Remover as Layers
	while (this.layers.length > 0) {
		var i = this.layers.pop();
		i.setMap(null);
	}
}

L2SGMaps.prototype.addInfos = function(itemId, altText, infoText) {
	/*//Remove os pontos/marcas
	while (this.markees.length > 0) {
		var m = this.markees.pop();
		m.setVisible(false);
	}*/
	var marker = new MarkerWithLabel({
		position: new google.maps.LatLng(0,0),
		draggable: false,
		raiseOnDrag: false,
		map: this.Map,
		labelContent: altText,
		labelAnchor: new google.maps.Point(30, 20),
		labelClass: "labels", // the CSS class for the label
		labelStyle: {opacity: 1.0},
		icon: "1x1.png",
		visible: false
	});

	for(var idx in this.linePaths) {
		var polygon = this.linePaths[idx];
		if(polygon.__itemId == itemId) {
			google.maps.event.addListener(poly, "mousemove", function(event) {
				marker.setPosition(event.latLng);
				marker.setVisible(true);
			});
			google.maps.event.addListener(poly, "mouseout", function(event) {
				marker.setVisible(false);
			});
		}
	}
}

L2SGMaps.prototype.setZoom = function (zoomNumber) {
  this.Map.setZoom(zoomNumber);
}

L2SGMaps.prototype.setCenter = function (address) {
  var centerLatLng = this.extractCoords(address);
  if(centerLatLng) {
    this.Map.setCenter(centerLatLng);
  } else {
    this.Geo.geocode({'address': address},
      function(results, status){
        if (status == google.maps.GeocoderStatus.OK) {
		  __gMaps.Map.setCenter(results[0].geometry.location);
        } else {
          __gMaps.doErrorEvent(status, address);
		}
      }
    );
  }
}

L2SGMaps.prototype.setMapType = function (mType) {
  var v;
  switch(parseInt(mType)) {
    case 'E':
    case 1: v = google.maps.MapTypeId.ROADMAP; break;
	case 'S':
    case 2: v = google.maps.MapTypeId.SATELLITE; break;
	case 'M':
    case 3: v = google.maps.MapTypeId.HYBRID; break;
	case 'T':
    case 4: v = google.maps.MapTypeId.TERRAIN; break;
  }

  this.Map.setMapTypeId(v);
}

L2SGMaps.prototype.processQueue = function (buffer) {
	if(!buffer.queue) return;
  	for(var i = 0; i < buffer.queue.length; i++) {
		var queueItem = buffer.queue[i];
		var fParams = new Array();
		for(var j = 0; j < queueItem.params.length; j++) {
			var p = queueItem.params[j];
			if(p instanceof Boolean) {
				//
			} else if(typeof p == 'string') {
				p = p.replace(/\n/g, '\\n');
				if(p.indexOf('"') > 0) {
					p = "'" + p + "'";
				} else {
					p = '"' + p + '"';
				}
			} else if(p instanceof Date) {
				p = ' new Date(' + p.getTime() + ')';
			} else {
				p = '"' + p + '"';
			}
			fParams.push(p);
		}
		var code = 'document.controller.' + queueItem.name + '(' + fParams.join(', ') + ');';
		eval(code);
	}
}

var generateIconCache = {};
function generateIcon(number, callback) {
  if (generateIconCache[number] !== undefined) {
    callback(generateIconCache[number]);
  }

  var fontSize = 16,
    imageWidth = imageHeight = 35;

  if (number >= 1000) {
    fontSize = 10;
    imageWidth = imageHeight = 55;
  } else if (number < 1000 && number > 100) {
    fontSize = 14;
    imageWidth = imageHeight = 45;
  }

  var svg = d3.select(document.createElement('div')).append('svg')
    .attr('viewBox', '0 0 54.4 54.4')
    .append('g')

  var circles = svg.append('circle')
    .attr('cx', '27.2')
    .attr('cy', '27.2')
    .attr('r', '21.2')
    .style('fill', '#2063C6');

  var path = svg.append('path')
    .attr('d', 'M27.2,0C12.2,0,0,12.2,0,27.2s12.2,27.2,27.2,27.2s27.2-12.2,27.2-27.2S42.2,0,27.2,0z M6,27.2 C6,15.5,15.5,6,27.2,6s21.2,9.5,21.2,21.2c0,11.7-9.5,21.2-21.2,21.2S6,38.9,6,27.2z')
    .attr('fill', '#FFFFFF');

  var text = svg.append('text')
    .attr('dx', 27)
    .attr('dy', 32)
    .attr('text-anchor', 'middle')
    .attr('style', 'font-size:' + fontSize + 'px; fill: #FFFFFF; font-family: Arial, Verdana; font-weight: bold')
    .text(number);

  var svgNode = svg.node().parentNode.cloneNode(true),
    image = new Image();

  d3.select(svgNode).select('clippath').remove();

  var xmlSource = (new XMLSerializer()).serializeToString(svgNode);

  image.onload = (function(imageWidth, imageHeight) {
    var canvas = document.createElement('canvas'),
      context = canvas.getContext('2d'),
      dataURL;

    d3.select(canvas)
      .attr('width', imageWidth)
      .attr('height', imageHeight);

    context.drawImage(image, 0, 0, imageWidth, imageHeight);

    dataURL = canvas.toDataURL();
    generateIconCache[number] = dataURL;

    callback(dataURL);
  }).bind(this, imageWidth, imageHeight);

  image.src = 'data:image/svg+xml;base64,' + btoa(encodeURIComponent(xmlSource).replace(/%([0-9A-F]{2})/g, function(match, p1) {
    return String.fromCharCode('0x' + p1);
  }));
}

L2SGMaps.prototype.loadKmlLayer2 = function(itemId, basePath, images, zoomMin, onClickEvent, onDblClickEvent) {
	if(typeof images == 'string') {
		images = JSON.parse(images);
	}
	var _gmaps = this;
	google.maps.event.addListener(this.Map, 'bounds_changed', function(event) {
		if(_gmaps.Map.getZoom() >= zoomMin) {
			var bounds = _gmaps.Map.getBounds();
			var center = _gmaps.Map.getCenter();
			var northEast = bounds.getNorthEast();
			var southWest = bounds.getSouthWest();
			var imagesPendding = [];
			for(var idx in images) {
				var img = images[idx];
				var fitY = false;
				var fitX = false;
				if (((img.north <= northEast.lat()) && (img.north >= southWest.lat())) || ((img.south <= northEast.lat()) && (img.south >= southWest.lat()))) {
					fitY = true;
				}
				if(((img.east <= northEast.lng()) && (img.east >= southWest.lng())) || ((img.west <= northEast.lng()) && (img.west >= southWest.lng()))) {
					fitX = true;
				}
				if(fitX && fitY) {
					if(!img.loaded) {
						console.log('Loading KML image: ' + basePath + img.imagem + ' (' + img.south + ';' + img.west + ', ' + img.north + ';' + img.east + ')');
						_gmaps.drawOverlay(img.imagem, img.south + ';' + img.west, img.north + ';' + img.east, basePath + img.imagem, onClickEvent, onDblClickEvent);
						img.loaded = true;
					}
				} else {
					imagesPendding.push(img);
				}
			}
			images = imagesPendding;
		}
	});
}

L2SGMaps.prototype.drawOverlay = function (itemId, from, to, srcImage, onClickEvent, onDblClickEvent) {
	var fromCoords = this.extractCoords(from);
	if(!fromCoords) {
		fromCoords = this.getAddressCoord(from);
	}

	var toCoords = this.extractCoords(to);
	if(!toCoords) {
		toCoords = this.getAddressCoord(to);
	}
	//
	if(toCoords && fromCoords) {
		var bounds = new google.maps.LatLngBounds(fromCoords, toCoords);
		var overlay = new L2SImgOverlay(bounds, srcImage, this.Map); 
		if(!isNullOrEmpty(onClickEvent)) {
			google.maps.event.addListener(overlay, 'click', function(event) {
				callFlow(onClickEvent, [itemId, event.latLng.lat(), event.latLng.lng()]) 
			});
		}

		if(!isNullOrEmpty(onDblClickEvent)) {
			google.maps.event.addListener(overlay, 'dblclick', function(event) {
										callFlow(onDblClickEvent, [itemId, event.latLng.lat(), event.latLng.lng()]);
									});
		}
		overlay.__layer_name = itemId;
		this.layers.push(overlay);
	} else {
		this.doErrorEvent(404, 'Endereço não encontrado. Origem: "' + from + '", destino "' + to + '".'); 
	}
}

L2SImgOverlay.prototype = new google.maps.OverlayView();

// [START region_constructor]
/** @constructor */
function L2SImgOverlay(bounds, image, map) {

  // Initialize all properties.
  this.bounds_ = bounds;
  this.image_ = image;
  this.map_ = map;

  // Define a property to hold the image's div. We'll
  // actually create this div upon receipt of the onAdd()
  // method so we'll leave it null for now.
  this.div_ = null;

  // Explicitly call setMap on this overlay.
  this.setMap(map);
}
// [END region_constructor]

// [START region_attachment]
/**
 * onAdd is called when the map's panes are ready and the overlay has been
 * added to the map.
 */
L2SImgOverlay.prototype.onAdd = function() {

  var div = document.createElement('div');
  div.style.borderStyle = 'none';
  div.style.borderWidth = '0px';
  div.style.position = 'absolute';

  // Create the img element and attach it to the div.
  var img = document.createElement('img');
  img.src = this.image_;
  img.style.width = '100%';
  img.style.height = '100%';
  img.style.position = 'absolute';
  div.appendChild(img);

  this.div_ = div;

  // Add the element to the "overlayLayer" pane.
  var panes = this.getPanes();
  panes.overlayLayer.appendChild(div);
};
// [END region_attachment]

// [START region_drawing]
L2SImgOverlay.prototype.draw = function() {

  // We use the south-west and north-east
  // coordinates of the overlay to peg it to the correct position and size.
  // To do this, we need to retrieve the projection from the overlay.
  var overlayProjection = this.getProjection();

  // Retrieve the south-west and north-east coordinates of this overlay
  // in LatLngs and convert them to pixel coordinates.
  // We'll use these coordinates to resize the div.
  var sw = overlayProjection.fromLatLngToDivPixel(this.bounds_.getSouthWest());
  var ne = overlayProjection.fromLatLngToDivPixel(this.bounds_.getNorthEast());

  // Resize the image's div to fit the indicated dimensions.
  var div = this.div_;
  div.style.left = sw.x + 'px';
  div.style.top = ne.y + 'px';
  div.style.width = (ne.x - sw.x) + 'px';
  div.style.height = (sw.y - ne.y) + 'px';
};
// [END region_drawing]

// [START region_removal]
// The onRemove() method will be called automatically from the API if
// we ever set the overlay's map property to 'null'.
L2SImgOverlay.prototype.onRemove = function() {
  this.div_.parentNode.removeChild(this.div_);
  this.div_ = null;
};
// [END region_removal]

/*L2MAKER*/